<?php

App::uses('BuzzSourceAppModel', 'BuzzSource.Model');

class BookingApi extends BuzzSourceAppModel {

/**
 * Get vouchers for an activity type at a specific site
 *
 * @param int $siteId Location/Site ID
 * @param int $activityType ID of activity type
 * @return array
 */
	public function getVouchersBySite($siteId, $activityType) {
		$result = $this->find('all', array(
			'method' => 'GetVouchersBySite',
			'conditions' => array(
				'siteId' => $siteId,
				'activityType' => 1
			)
		));

		return !empty($result['BookingApi']['d']) ? $this->_processVouchers($result['BookingApi']['d']) : false;
	}

/**
 * Process vouchers data from the API
 *
 * @param array $data Response from API
 * @return array
 */
	protected function _processVouchers($data) {
		$result = [];

		foreach ($data as $voucherProvider) {
			$vouchers = [];
			if (isset($voucherProvider['Vouchers'])) {
				foreach ($voucherProvider['Vouchers'] as $voucher) {
					$vouchers[$voucher['VoucherId']] = array(
						'id' => $voucher['VoucherId'],
						'name' => $voucher['VoucherName'],
						'description' => isset($voucher['VoucherDescription']) ? $voucher['VoucherDescription'] : '',
						'participants' => $voucher['Participants'],
						'code_type' => trim($voucher['NumberType']),
						'code_location' => trim($voucher['NumberLocation']),
						'requires_validation' => $voucher['RequiresValidation'],
						'requires_expiry_date' => $voucher['RequiresExpiryDate'] ? true : false,
						'activity_api_id' => $voucher['BookingTypeId'],
						'is_off_peak_only' => $voucher['IsOffPeakOnly'] ? true : false
					);
				}
			}

			$result[$voucherProvider['VoucherProviderId']] = array(
				'VoucherProvider' => array(
					'id' => $voucherProvider['VoucherProviderId'],
					'name' => $voucherProvider['VoucherProviderName'],
					'logo_url' => $voucherProvider['VoucherProviderLogoUrl'],
					'Voucher' => $vouchers
				)
			);

		}

		return $result;
	}

/**
 * Check a 'Bodyflight' voucher code
 *
 * @param string $code Voucher code
 * @param string $expiryDate Voucher expiry date
 * @return array|bool
 */
	public function checkVoucherCode($code, $expiryDate) {
		$result = $this->find('all', array(
			'method' => 'CheckBodyFlightVoucherCode',
			'conditions' => array(
				'voucherCode' => urlencode($code),
				'expiryDate' => $expiryDate
			)
		));

		return !empty($result['BookingApi']['d']) ? $this->_processCheckVoucherCode($result['BookingApi']['d']) : false;
	}

/**
 * Process the response from checkVoucherCode.
 *
 * @param array $data
 * @return array
 */
	protected function _processCheckVoucherCode(array $data) {
		$result = [];

		if (
			!empty($data['Status'])
			&& strtolower($data['Status']) === 'ok'
			&& !empty($data['Activities'])
		) {
			foreach ($data['Activities'] as $activity) {
				$activities[] = array(
					'api_activity_reference' => $activity['ActivityId'],
					'api_package_reference' => $activity['BookingTypeId'],
					'quantity' => $activity['RequiredQuantity'],
				);
			}
			$result = array(
				'Voucher' => array(
					'participants' => $data['Participants'],
					'description' => $data['VoucherDescription'],
					'Activity' => $activities
				)
			);
		}

		return $result;
	}

/**
 * Check a third-party voucher code
 *
 * @param string $code Voucher code
 * @param string $expiryDate Voucher expiry date
 * @return array|bool
 */
	public function checkThirdPartyVoucherCode($code, $expiryDate) {
		$result = $this->find('all', array(
			'method' => 'CheckVoucherCode',
			'conditions' => array(
				'voucherCode' => urlencode($code),
				'expiryDate' => $expiryDate
			)
		));

		if (!empty($result['BookingApi']['d'])) {
			return array(
				'Voucher' => array(
					'description' => $result['BookingApi']['d']['VoucherDescription']
				)
			);
		}

		return false;
	}

/**
 * Checks that the voucher has not already been used.
 *
 * @param int $activityId
 * @param int $voucherProviderId
 * @param string $voucherCode
 * @return bool
 */
	public function checkVoucherAvailable($activityId, $voucherProviderId, $voucherCode) {
		$result = $this->find('all', array(
			'method' => 'IsVoucherUsed',
			'conditions' => array(
				'activityId' => $activityId,
				'vouchersupplierId' => $voucherProviderId,
				'voucherno' => urlencode($voucherCode)
			)
		));

		return empty($result['BookingApi']['d']);
	}

/**
 * Get available dates for bookings based on current booking items.
 *
 * @param int $siteId
 * @param string $fromDate
 * @param string $toDate
 * @param array $items Booking items
 * @param bool $concurrent Set to false if the booking items take place in series rather than at the same time
 * @param bool $offPeakOnly
 * @return array
 */
	public function getDates($siteId, $fromDate, $toDate, array $items = [], $concurrent = true, $offPeakOnly = false) {
		// We Need to build up an array of booking items and their
		// quantities to pass to the API.
		$params = [];
		$i = 0;
		foreach ($items as $bookingTypeId => $quantity) {
			++$i;
			$params['bookingtype' . $i] = $bookingTypeId;
			$params['quantity' . $i] = $quantity;
		}

		$result = $this->find('all', array(
			'method' => 'GetAvailableDates',
			'conditions' => array(
				'siteId' => $siteId,
				'fromDate' => $fromDate,
				'toDate' => $toDate,
				'concurrent' => $concurrent === true ? 'true' : 'false',
				'uniquetimes' => 'true'
			) + $params
		));

		return !empty($result['BookingApi']['d']) ? $this->_processDates($result['BookingApi']['d']) : false;
	}

/**
 * Process dates returned by the API
 *
 * @param array $data
 * @param bool $offPeakOnly
 * @return array
 */
	protected function _processDates($data, $offPeakOnly = false) {
		$result = [];

		// Set some date/time variables ready for use later so that we don't
		// need to keep calculating these.
		$today = date('d/m/Y');
		$twoHoursFromNow = date('H:i', strtotime('+2 hours'));

		foreach ($data as $date) {
			$times = [];

			foreach ($date['Itineraries'] as $itinerary) {
				$time = null;
				$checkIn = null;
				$peak = false;
				$activities = [];

				foreach ($itinerary as $activity) {
					$activities[] = array(
						'activity_api_id' => $activity['BookingTypeId'],
						'description' => $activity['Description'],
						'check_in' => $activity['CheckInMins'],
						'from_time' => $activity['FromTime'],
						'to_time' => $activity['ToTime'],
						'peak' => (bool)$activity['IsPeak'],
						'experience_id' => $activity['ExperienceId']
					);

					if ($time === null || $activity['FromTime'] < $time) {
						$time = $activity['FromTime'];
					}

					if ($checkIn === null || $activity['CheckInMins'] > $checkIn) {
						$checkIn = $activity['CheckInMins'];
					}

					$peak = $activity['IsPeak'] === true ? true : $peak;

				}

				// We only want to keep times 2 hours in the future from now.
				if (
					((string)$date['Date'] !== $today || $time >= $twoHoursFromNow)
					&& ($offPeakOnly === false || $peak === false)
				) {
					$times[] = array(
						'time' => $time,
						'check_in' => $checkIn,
						'activities' => $activities,
						'peak' => $peak
					);
				}

			}

			if (!empty($times)) {
				// Sort the times.
				usort($times, function ($a, $b) {
					return $a['time'] > $b['time'];
				});
				// Store in the results array.
				$result[] = array(
					'date' => $date['Date'],
					'times' => $times
				);
			}

		}

		return $result;
	}

}
